
# q-expansion of newform 1152.4.d.i, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1152
weight = 4
poly_data = [9, 0, 7, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 16, 0, 4], 3], [[0, 40, 0, 4], 3], [[28, 0, 8, 0], 1]]

hecke_ring_character_values = [[127, [1, 0, 0, 0]], [901, [-1, 0, 0, 0]], [641, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, -1, 1, 0], [-8, 0, 0, 1], [0, 1, 4, 0], [0, 9, 2, 0], [18, 0, 0, 4], [0, -17, 4, 0], [128, 0, 0, 2], [0, -19, 9, 0], [-40, 0, 0, -13], [0, -17, 4, 0], [-218, 0, 0, 20], [0, -89, 4, 0], [112, 0, 0, -14], [0, 43, 15, 0], [0, 81, 0, 0], [0, 81, 0, 0], [0, -57, -48, 0], [1024, 0, 0, -2], [-330, 0, 0, 48], [248, 0, 0, 59], [0, 97, 28, 0], [266, 0, 0, -8], [-610, 0, 0, -88], [0, -153, -77, 0], [-856, 0, 0, -73], [0, 61, -8, 0], [0, 341, -42, 0], [-114, 0, 0, -56], [2152, 0, 0, -35], [0, -327, -48, 0], [-1594, 0, 0, -68], [0, 71, -112, 0], [0, -53, 53, 0], [312, 0, 0, 69], [0, 377, 140, 0], [0, 419, -52, 0], [-528, 0, 0, -60], [0, 745, 23, 0], [0, -323, 112, 0], [0, -77, 234, 0], [-1568, 0, 0, -20], [-914, 0, 0, 272], [0, -245, 131, 0], [1064, 0, 0, 119], [0, 19, -32, 0], [-1720, 0, 0, 161], [0, -407, 100, 0], [0, -129, -50, 0], [-102, 0, 0, -8], [864, 0, 0, 360], [-4162, 0, 0, -184], [0, -915, 84, 0], [-642, 0, 0, -160], [1168, 0, 0, -416], [0, 545, -227, 0], [-4248, 0, 0, 81], [0, -369, -286, 0], [3434, 0, 0, -224], [0, 427, -272, 0], [0, -613, -233, 0], [0, -969, -240, 0], [-2480, 0, 0, -284], [5018, 0, 0, -240], [0, 2029, 93, 0], [0, -2525, -56, 0], [5410, 0, 0, -80], [0, -1011, -12, 0], [0, 61, -224, 0], [6270, 0, 0, 136], [-1152, 0, 0, 486], [-3144, 0, 0, -525], [0, -1637, 328, 0], [0, 1135, 364, 0], [6368, 0, 0, 356], [0, -853, 237, 0], [0, -2159, 184, 0], [-3694, 0, 0, -468], [2026, 0, 0, 424], [0, -835, 836, 0], [0, -1809, -430, 0], [-1200, 0, 0, -426], [2414, 0, 0, 144], [2472, 0, 0, 483], [0, -1047, -588, 0], [8610, 0, 0, 308], [506, 0, 0, 56], [0, -1123, 355, 0], [-2968, 0, 0, 713], [0, -459, 180, 0], [-8240, 0, 0, 58], [3416, 0, 0, 5], [0, -519, -312, 0], [0, 547, 208, 0], [5472, 0, 0, -882], [0, 121, 425, 0], [16550, 0, 0, 292], [0, -3013, -100, 0], [0, -843, 46, 0], [0, 1047, 804, 0], [0, -951, 395, 0], [0, -147, -228, 0], [502, 0, 0, -1068], [0, 5395, 232, 0], [12946, 0, 0, 560], [0, 1933, 424, 0], [-2002, 0, 0, 424], [7200, 0, 0, -162], [4710, 0, 0, -800], [22952, 0, 0, -331], [0, -1845, -576, 0], [-17574, 0, 0, 80], [0, 1091, 80, 0], [5176, 0, 0, -467], [-1022, 0, 0, 1124], [0, -489, -1020, 0], [6240, 0, 0, -222], [0, 485, 495, 0], [0, 2925, 360, 0], [0, 2527, -1052, 0], [11550, 0, 0, 1184], [0, -1221, -775, 0], [0, 1693, 76, 0], [0, 2563, 460, 0], [0, -3279, -359, 0], [0, -517, 1638, 0], [-13872, 0, 0, 1590], [-17704, 0, 0, 737], [0, -791, -966, 0], [0, -1357, -1576, 0], [29072, 0, 0, 344], [-2152, 0, 0, -2233], [0, 6883, 134, 0], [-810, 0, 0, 1948], [-3218, 0, 0, 384], [0, -3009, 1127, 0], [0, -2037, -228, 0], [0, 3933, -113, 0], [-2842, 0, 0, 1500], [0, 4291, -188, 0], [0, 763, 1715, 0], [-22952, 0, 0, -1127], [0, 1165, 88, 0], [0, 4853, -1434, 0], [5856, 0, 0, 834], [0, -2045, -548, 0], [10454, 0, 0, 468], [0, 311, -1780, 0], [35104, 0, 0, 796], [0, -2687, 232, 0], [-15506, 0, 0, 240], [0, -3217, 1388, 0], [-3024, 0, 0, 1728], [0, -593, 220, 0], [3552, 0, 0, 1608], [-9192, 0, 0, 897], [4034, 0, 0, 1500], [-9210, 0, 0, -608], [0, -339, -469, 0], [0, -5271, 912, 0], [25622, 0, 0, 292], [23112, 0, 0, -2133], [0, -6067, 1292, 0], [-51310, 0, 0, 508], [-16528, 0, 0, 608], [10360, 0, 0, -17], [0, 4759, 1720, 0], [-20178, 0, 0, -1224], [0, 10643, 457, 0], [0, 685, -3560, 0], [0, 5277, -1356, 0], [-44400, 0, 0, 168], [-22874, 0, 0, 1984], [14648, 0, 0, -1813], [-29910, 0, 0, -1024], [0, 599, 560, 0], [0, 1371, 1933, 0], [2120, 0, 0, -2173], [0, 13469, -196, 0], [9944, 0, 0, -685], [0, -12095, 652, 0], [0, 3967, 854, 0], [4730, 0, 0, -512], [3648, 0, 0, 1668], [0, 6627, -1135, 0], [0, -2059, 154, 0], [0, -5365, -1984, 0], [-26038, 0, 0, -2728], [-8832, 0, 0, 672], [-4286, 0, 0, -272], [0, 3833, 4208, 0], [0, 871, -2672, 0], [0, 5681, 787, 0], [0, 5109, 528, 0], [-8294, 0, 0, -2224], [-17182, 0, 0, 3408], [0, -7395, 1850, 0], [18910, 0, 0, -112], [9200, 0, 0, 2828], [0, 4345, 3335, 0], [-30520, 0, 0, -379], [0, 7399, -1038, 0], [-20526, 0, 0, 912], [0, -11439, 540, 0], [0, -12667, 925, 0], [29608, 0, 0, -3251], [0, 17681, 236, 0], [-28102, 0, 0, 2088], [0, 4811, 3692, 0], [-14850, 0, 0, 3008], [0, 15659, 433, 0], [-64920, 0, 0, 1491], [0, 1605, 2928, 0], [2592, 0, 0, 306], [41498, 0, 0, -368], [47192, 0, 0, 1391], [-51598, 0, 0, -284], [72336, 0, 0, 156], [0, 1641, 1087, 0], [0, -5481, 938, 0], [-36600, 0, 0, -2985], [1602, 0, 0, 5980], [-57592, 0, 0, 845], [0, 13301, -3460, 0], [0, -9973, -1084, 0], [-7094, 0, 0, 2504], [-45616, 0, 0, -2830], [-45016, 0, 0, -2473], [0, -12991, 1064, 0], [0, -3363, -3914, 0], [0, 1919, 1700, 0], [-27832, 0, 0, 5441], [-73286, 0, 0, 176], [0, -20897, 364, 0], [-69584, 0, 0, 130], [-33266, 0, 0, 4120], [0, -12957, -947, 0], [0, 16249, 700, 0], [-20256, 0, 0, 1218], [0, -3271, 4700, 0], [0, 10131, -2856, 0], [-53064, 0, 0, 3105], [0, -4331, 1972, 0], [-36306, 0, 0, 2624], [38208, 0, 0, -3318], [15256, 0, 0, 4987], [0, -927, 2088, 0], [0, 22943, -424, 0], [34816, 0, 0, -284], [0, -12623, 2750, 0], [117790, 0, 0, 264], [45024, 0, 0, -318], [-9306, 0, 0, -5904], [0, -6343, -1787, 0], [0, 3629, 6776, 0], [0, -10237, 2218, 0], [0, 5735, 2456, 0], [0, -15625, -1889, 0], [25546, 0, 0, 3408], [68296, 0, 0, -1103], [0, -6843, -1344, 0], [0, -853, -4814, 0], [0, -10603, 2228, 0], [-4318, 0, 0, -1748], [0, -6025, -1384, 0], [0, 12181, -5137, 0], [9610, 0, 0, -664], [0, -9017, -4460, 0], [0, 3059, 2853, 0], [0, -17675, 1624, 0], [0, 15415, 4600, 0], [92470, 0, 0, 1984], [-6888, 0, 0, 339], [-1262, 0, 0, -2432], [1800, 0, 0, 3159], [0, 4113, 3276, 0], [0, 22745, 80, 0], [16278, 0, 0, 1088], [0, 1401, -5160, 0], [6288, 0, 0, -8238], [85576, 0, 0, -2309], [22944, 0, 0, -3138], [0, -21549, -552, 0], [0, 6399, -8892, 0], [45248, 0, 0, -5908], [-15842, 0, 0, 7288], [0, -21809, -1651, 0], [107736, 0, 0, 1833], [81214, 0, 0, -2968], [0, 3773, 3351, 0], [0, -23639, -992, 0], [59050, 0, 0, -2712], [0, 645, 4416, 0], [0, 10113, -7716, 0], [0, 23821, 2715, 0], [31432, 0, 0, 2065], [0, -12689, 4067, 0], [0, -9519, -1140, 0], [0, 11767, 1708, 0], [92490, 0, 0, -3232], [0, -14911, 1723, 0], [79288, 0, 0, 3355]]
